import datetime
import re
import time
import warnings
from functools import partialmethod

import jdatetime
from django.conf import settings
from django.core import exceptions
from django.db import models
from django.utils import timezone
from django.utils.encoding import smart_str
from django.utils.translation import gettext as _

from django_jalali import forms

ansi_date_re = re.compile(r'^\d{4}-\d{1,2}-\d{1,2}$')


class jQuerySet(models.QuerySet):
    """we need to rewrite this class to handle year filter"""

    def filter(self, *args, **kwargs):
        """if filter is year we divide to __gte and __lte"""
        new_kwargs = {}
        for k in kwargs:
            if '__year' in k:
                filed_name = k.split('__year')
                first_year = jdatetime.datetime(int(kwargs[k]), 1, 1)
                new_kwargs['%s__gte' % filed_name[0]] = jdatetime.datetime(
                    int(kwargs[k]), 1, 1)
                last_day = 29
                if first_year.isleap():
                    last_day = 30
                new_kwargs['%s__lte' % filed_name[0]] = jdatetime.datetime(
                    int(kwargs[k]), 12, last_day, 23, 59, 59)
            else:
                new_kwargs[k] = kwargs[k]
        return super().filter(*args, **new_kwargs)


jManager = models.Manager.from_queryset(jQuerySet)


class jDateField(models.DateField):
    description = _("Date (without time)")
    empty_strings_allowed = False
    default_error_messages = {
        'invalid': _('Enter a valid date in YYYY-MM-DD format.'),
        'invalid_date': _('Invalid date: %s'),
    }

    def __init__(
        self, verbose_name=None, name=None, auto_now=False,
        auto_now_add=False, default=models.NOT_PROVIDED, **kwargs
    ):

        self.auto_now, self.auto_now_add = auto_now, auto_now_add
        # HACKs : auto_now_add/auto_now should be
        # done as a default or a pre_save.
        if auto_now or auto_now_add:
            kwargs['editable'] = False
            kwargs['blank'] = True

        # Convert datetime.date default value to jdatetime.date
        if isinstance(default, datetime.date):
            default = jdatetime.date.fromgregorian(date=default)

        models.Field.__init__(self, verbose_name, name, default=default, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()

        # Convert jdatetime.date default value to datetime.date
        if 'default' in kwargs and isinstance(kwargs['default'], jdatetime.date):
            kwargs['default'] = kwargs['default'].togregorian()

        return name, path, args, kwargs

    def get_internal_type(self):
        return "DateField"

    @classmethod
    def parse_date(cls, date_obj):
        "Take a datetime object and convert it to jalali date"

        if isinstance(date_obj, datetime.datetime):
            return jdatetime.date.fromgregorian(date=date_obj.date())
        if isinstance(date_obj, datetime.date):
            return jdatetime.date.fromgregorian(date=date_obj)

        if not ansi_date_re.search(date_obj):
            raise exceptions.ValidationError(cls.default_error_messages['invalid'])
        # Now that we have the date string in YYYY-MM-DD format, check to make
        # sure it's a valid date.
        # We could use time.strptime here and catch errors, but datetime.date
        # produces much friendlier error messages.
        year, month, day = map(int, date_obj.split('-'))
        try:
            if year > 1500:
                return jdatetime.date.fromgregorian(
                    date=datetime.date(year, month, day))
            else:
                return jdatetime.date(year, month, day)
        except ValueError as e:
            msg = cls.default_error_messages['invalid_date'] % _(str(e))
            raise exceptions.ValidationError(msg)

    def from_db_value(self, value, expression, connection):
        if value is None:
            return value
        return self.parse_date(value)

    def to_python(self, value):
        if value is None:
            return value

        if isinstance(value, jdatetime.datetime):
            return value.date()
        if isinstance(value, jdatetime.date):
            return value
        return self.parse_date(value)

    def pre_save(self, model_instance, add):
        if self.auto_now or (self.auto_now_add and add):
            value = jdatetime.date.today()
            setattr(model_instance, self.attname, value)
            return value
        else:
            return super().pre_save(model_instance, add)

    def contribute_to_class(self, cls, name):
        super().contribute_to_class(cls, name)
        if not self.null:
            setattr(
                cls,
                'get_next_by_%s' % self.name,
                partialmethod(cls._get_next_or_previous_by_FIELD, field=self, is_next=True)
            )
            setattr(
                cls,
                'get_previous_by_%s' % self.name,
                partialmethod(cls._get_next_or_previous_by_FIELD, field=self, is_next=False)
            )

    def get_prep_lookup(self, lookup_type, value):
        """this class dosn't work in month and day search !"""
        # For "__month", "__day", and "__week_day" lookups, convert the value
        # to an int so the database backend always sees a consistent type.

        if lookup_type in ('exact', 'gt', 'gte', 'lt', 'lte'):
            prep = self.get_prep_value(value)
            if type(prep) == datetime.datetime or type(prep) == datetime.date:
                return prep
            return prep.togregorian()

        elif lookup_type in ('range', 'in'):
            return [self.get_prep_value(v) for v in value]
        elif lookup_type == 'year':
            # this else never happen !
            try:
                return int(value)
            except ValueError:
                raise ValueError(
                    "The __year lookup type requires an integer argument")

        if lookup_type in ('month', 'day', 'week_day'):
            raise ValueError(
                "jDateField doesn't work with month, day and week_day !")

        return super().get_prep_lookup(lookup_type, value)

    def get_prep_value(self, value):
        return self.to_python(value)

    def get_db_prep_value(self, value, connection, prepared=False):
        # Casts dates into the format expected by the backend
        if not prepared:
            value = self.get_prep_value(value)

        if isinstance(value, jdatetime.datetime):
            value = value.togregorian().date()
        if isinstance(value, jdatetime.date):
            value = value.togregorian()

        return connection.ops.adapt_datefield_value(value)

    def value_to_string(self, obj):
        value = self.value_from_object(obj)
        if value is None:
            date_string = ''
        else:
            date_string = smart_str(value)
        return date_string

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.jDateField}
        kwargs.update(defaults)
        return super().formfield(**kwargs)


class jDateTimeField(models.DateTimeField):
    default_error_messages = {
        'invalid': _(
            u'Enter a valid date/time in '
            u'YYYY-MM-DD HH:MM[:ss[.uuuuuu]] format.'),
    }
    description = _("Date (with time)")

    def __init__(
        self, verbose_name=None, name=None, auto_now=False,
        auto_now_add=False, default=models.NOT_PROVIDED, **kwargs
    ):

        self.auto_now, self.auto_now_add = auto_now, auto_now_add
        # HACKs : auto_now_add/auto_now should be
        # done as a default or a pre_save.
        if auto_now or auto_now_add:
            kwargs['editable'] = False
            kwargs['blank'] = True

        # Convert datetime.datetime default value to jdatetime.datetime
        if isinstance(default, datetime.date):
            default = jdatetime.datetime.fromgregorian(datetime=default)

        models.Field.__init__(self, verbose_name, name, default=default, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()

        # Convert jdatetime.datetime default value to datetime.datetime
        default = kwargs.get('default')
        if default and isinstance(default, (jdatetime.datetime)):
            kwargs['default'] = default.togregorian()

        return name, path, args, kwargs

    def get_internal_type(self):
        return "DateTimeField"

    @classmethod
    def parse_date(cls, datetime_obj):
        "Take a datetime object and convert it to jalali date"

        if isinstance(datetime_obj, datetime.datetime):
            try:
                if datetime_obj.year < 1700:
                    return jdatetime.datetime(
                        datetime_obj.year, datetime_obj.month,
                        datetime_obj.day, datetime_obj.hour,
                        datetime_obj.minute, datetime_obj.second,
                        datetime_obj.microsecond, datetime_obj.tzinfo)
                else:
                    return jdatetime.datetime.fromgregorian(
                        datetime=datetime_obj)
            except ValueError:
                raise exceptions.ValidationError(
                    cls.default_error_messages['invalid'])
        if isinstance(datetime_obj, datetime.date):
            try:
                if datetime_obj.year < 1700:
                    return jdatetime.datetime(datetime_obj.year,
                                              datetime_obj.month,
                                              datetime_obj.day)
                else:
                    return jdatetime.datetime.fromgregorian(date=datetime_obj)
            except ValueError:
                raise exceptions.ValidationError(
                    cls.default_error_messages['invalid'])

        # Attempt to parse a datetime:
        datetime_obj = smart_str(datetime_obj)
        if not datetime_obj:
            return None
        # split usecs, because they are not recognized by strptime.
        if '.' in datetime_obj:
            try:
                datetime_obj, usecs = datetime_obj.split('.')
                if '+' in usecs:
                    usecs, tz = usecs.split('+')
                usecs = int(usecs)
            except ValueError:
                raise exceptions.ValidationError(
                    cls.default_error_messages['invalid'])
        else:
            usecs = 0
        kwargs = {'microsecond': usecs}

        formats = [
            ('%Y-%m-%d %H:%M:%S', 6),
            ('%Y-%m-%d %H:%M', 5),
            ('%Y-%m-%d', 3),
        ]
        for format, c in formats:
            try:  # Seconds are optional, so try converting seconds first.
                t = time.strptime(datetime_obj, format)
                if t.tm_year > 1700:
                    return datetime.datetime(
                        *time.strptime(datetime_obj, format)[:c],
                        **kwargs)
                else:
                    return jdatetime.datetime(
                        *time.strptime(datetime_obj, format)[:c],
                        **kwargs)
            except ValueError:
                try:
                    return jdatetime.datetime.strptime(
                        datetime_obj,
                        '%Y-%m-%d %H:%M'
                    ).replace(**kwargs)
                except ValueError:
                    pass
        raise exceptions.ValidationError(cls.default_error_messages['invalid'])

    def from_db_value(self, value, expression, connection):
        if value is None:
            return value

        if value is not None and settings.USE_TZ:
            # Remove and set timezone information
            default_timezone = timezone.get_default_timezone()
            value = timezone.make_naive(value, default_timezone)
            value = timezone.make_aware(value, default_timezone)

        return self.parse_date(value)

    def to_python(self, value):
        if value is None:
            return value

        if isinstance(value, jdatetime.datetime):
            return value
        if isinstance(value, jdatetime.date):
            try:
                return jdatetime.datetime(value.year, value.month, value.day)
            except ValueError:
                raise exceptions.ValidationError(
                    self.error_messages['invalid'])
        return self.parse_date(value)

    def pre_save(self, model_instance, add):
        if self.auto_now or (self.auto_now_add and add):
            value = jdatetime.datetime.now()
            if value is not None and settings.USE_TZ and timezone.is_naive(value):
                default_timezone = timezone.get_default_timezone()
                value = timezone.make_aware(value, default_timezone)
            setattr(model_instance, self.attname, value)
            return value
        else:
            return super().pre_save(model_instance, add)

    def get_prep_value(self, value):
        value = self.to_python(value)
        if value is not None and settings.USE_TZ and timezone.is_naive(value):
            try:
                name = '%s.%s' % (self.model.__name__, self.name)
            except AttributeError:
                name = '(unbound)'
            warnings.warn(
                "DateTimeField %s received a naive datetime (%s)"
                " while time zone support is active." %
                (name, value),
                RuntimeWarning
            )
            default_timezone = timezone.get_default_timezone()
            value = timezone.make_aware(value, default_timezone)
        return value

    def get_db_prep_value(self, value, connection, prepared=False):
        # Casts dates into the format expected by the backend
        if not prepared:
            value = self.get_prep_value(value)

        if isinstance(value, jdatetime.datetime):
            value = value.togregorian()

        return connection.ops.adapt_datefield_value(value)

    def value_to_string(self, obj):
        value = self.value_from_object(obj)
        if value is None:
            date_string = ''
        else:
            date_string = smart_str(value)
        return date_string

    def contribute_to_class(self, cls, name):
        super().contribute_to_class(cls, name)
        if not self.null:
            setattr(
                cls,
                'get_next_by_%s' % self.name,
                partialmethod(cls._get_next_or_previous_by_FIELD, field=self, is_next=True)
            )
            setattr(
                cls,
                'get_previous_by_%s' % self.name,
                partialmethod(cls._get_next_or_previous_by_FIELD, field=self, is_next=False)
            )

    def get_prep_lookup(self, lookup_type, value):
        """this class dosn't work in month and day search !"""
        # For "__month", "__day", and "__week_day" lookups, convert the value
        # to an int so the database backend always sees a consistent type.

        if lookup_type in ('exact', 'gt', 'gte', 'lt', 'lte'):
            prep = self.get_prep_value(value)
            if type(prep) == datetime.datetime or type(prep) == datetime.date:
                return prep
            return prep.togregorian()

        elif lookup_type in ('range', 'in'):
            return [self.get_prep_value(v) for v in value]
        elif lookup_type == 'year':
            # this else never happen !
            try:
                return int(value)
            except ValueError:
                raise ValueError(
                    "The __year lookup type requires an integer argument")

        if lookup_type in ('month', 'day', 'week_day'):
            raise ValueError(
                "jDateTimeField doesn't work with month, day and week_day !")

        return super().get_prep_lookup(lookup_type, value)

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.jDateTimeField}
        kwargs.update(defaults)
        return super().formfield(**kwargs)



import jdatetime
from django.core import exceptions
from rest_framework import serializers

from django_jalali.db.models import (
    jDateField as jDateFieldModel, jDateTimeField as jDateTimeFieldModel,
)


class JDateField(serializers.DateField):
    def to_python(self, value):
        if value is None:
            return value

        if isinstance(value, jdatetime.datetime):
            return value.date()
        if isinstance(value, jdatetime.date):
            return value
        return jDateFieldModel.parse_date(value)

    # This method is used by DRF to bringing value back to python form
    def to_internal_value(self, value):
        return self.to_python(value)


class JDateTimeField(serializers.DateTimeField):
    def to_python(self, value):
        if value is None:
            return value

        if isinstance(value, jdatetime.datetime):
            return value
        if isinstance(value, jdatetime.date):
            try:
                return jdatetime.datetime(value.year, value.month, value.day)
            except ValueError:
                raise exceptions.ValidationError(
                    self.error_messages['invalid'])
        return jDateTimeFieldModel.parse_date(value)

    # This method is used by DRF to bringing value back to python form
    def to_internal_value(self, value):
        return self.to_python(value)

    # This method is used by DRF for representing object in string form
    # override this function with strftime if needed
    def to_representation(self, value):
        return str(value)



import re

import jdatetime
from django import forms
from django.core import exceptions, validators
from django.forms.utils import from_current_timezone, to_current_timezone
from django.utils.translation import gettext as _

from django_jalali.forms.widgets import jDateInput, jDateTimeInput


class jDateField(forms.Field):
    widget = jDateInput
    default_error_messages = {
        'invalid': _(u'Enter a valid date.'),
    }

    def __init__(self, input_formats=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.input_formats = input_formats

    def to_python(self, value):
        """
        Validates that the input can be converted to a date. Returns a Python
        datetime.date object.
        """
        if value in validators.EMPTY_VALUES:
            return None
        if isinstance(value, jdatetime.datetime):
            return value.date()
        if isinstance(value, jdatetime.date):
            return value

        groups = re.search(
            r'(?P<year>[\d]{1,4})-(?P<month>[\d]{1,2})-(?P<day>[\d]{1,2})',
            value,
        )
        try:
            return jdatetime.date(
                year=int(groups.group(1)),
                month=int(groups.group(2)),
                day=int(groups.group(3)),
            )

        except (ValueError, AttributeError):
            pass

        raise exceptions.ValidationError(self.error_messages['invalid'])


class jDateTimeField(forms.Field):
    widget = jDateTimeInput
    default_error_messages = {
        'invalid': _(u'Enter a valid date/time.'),
    }

    def __init__(self, input_formats=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.input_formats = input_formats

    def prepare_value(self, value):
        if isinstance(value, jdatetime.datetime):
            value = to_current_timezone(value)
        return value

    def to_python(self, value):
        """
        Validates that the input can be converted to a datetime. Returns a
        Python datetime.datetime object.
        """
        if value in validators.EMPTY_VALUES:
            return None
        if isinstance(value, jdatetime.datetime):
            return from_current_timezone(value)
        if isinstance(value, jdatetime.date):
            result = jdatetime.datetime(value.year, value.month, value.day)
            return from_current_timezone(result)
        if isinstance(value, list):
            # Input comes from a SplitDateTimeWidget, for example. So, it's two
            # components: date and time.
            if len(value) != 2:
                raise exceptions.ValidationError(self.error_messages['invalid'])
            if value[0] in validators.EMPTY_VALUES and value[1] in validators.EMPTY_VALUES:
                return None
            value = '%s %s' % tuple(value)

        groups = re.search(
            r'(?P<year>[\d]{1,4})-(?P<month>[\d]{1,2})-(?P<day>[\d]{1,2}) '
            r'(?P<hour>[\d]{1,2}):(?P<minute>[\d]{1,2})'
            r'(:(?P<second>[\d]{1,2}))?(.(?P<microsecond>[\d]{1,5}))?',
            value
        )
        try:
            microsecond = int(groups.group('microsecond') or 0)
            second = int(groups.group('second') or 0)
            result = jdatetime.datetime(
                year=int(groups.group('year')),
                month=int(groups.group('month')),
                day=int(groups.group('day')),
                hour=int(groups.group('hour')),
                minute=int(groups.group('minute')),
                second=second,
                microsecond=microsecond,
            )
            return from_current_timezone(result)

        except (ValueError, AttributeError):
            pass

        raise exceptions.ValidationError(self.error_messages['invalid'])



import jdatetime
from django.contrib import admin
from django.contrib.admin.options import IncorrectLookupParameters
from django.core.exceptions import ValidationError
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from django_jalali.db import models


class JDateFieldListFilter(admin.FieldListFilter):

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.field_generic = '%s__' % field_path
        self.date_params = {k: v for k, v in params.items()
                            if k.startswith(self.field_generic)}

        now = jdatetime.datetime.fromgregorian(datetime=timezone.now())
        if timezone.is_aware(now):
            now = timezone.localtime(now)

        if isinstance(field, models.jDateTimeField):
            today = now.replace(hour=0, minute=0, second=0, microsecond=0)
        else:
            today = now.date()

        tomorrow = today + jdatetime.timedelta(days=1)

        if today.month == 12:
            next_month = today.replace(year=today.year + 1, month=1, day=1)
        else:
            next_month = today.replace(month=today.month + 1, day=1)
        next_year = today.replace(year=today.year + 1, month=1, day=1)

        self.lookup_kwarg_since = '%s__gte' % field_path
        self.lookup_kwarg_until = '%s__lt' % field_path
        self.links = (
            (_('Any date'), {}),
            (_('Today'), {
                self.lookup_kwarg_since: today.strftime('%Y-%m-%d %H:%M:%S'),
                self.lookup_kwarg_until: tomorrow.strftime('%Y-%m-%d %H:%M:%S'),
            }),
            (_('Past 7 days'), {
                self.lookup_kwarg_since: (today - jdatetime.timedelta(days=7)).strftime('%Y-%m-%d %H:%M:%S'),
                self.lookup_kwarg_until: tomorrow.strftime('%Y-%m-%d %H:%M:%S'),
            }),
            (_('This month'), {
                self.lookup_kwarg_since: (today.replace(day=1)).strftime('%Y-%m-%d %H:%M:%S'),
                self.lookup_kwarg_until: next_month.strftime('%Y-%m-%d %H:%M:%S'),
            }),
            (_('This year'), {
                self.lookup_kwarg_since: (today.replace(month=1, day=1)).strftime('%Y-%m-%d %H:%M:%S'),
                self.lookup_kwarg_until: next_year.strftime('%Y-%m-%d %H:%M:%S'),
            }),
        )

        super().__init__(field, request, params, model, model_admin, field_path)

    def queryset(self, request, queryset):
        try:
            return queryset.filter(**self.used_parameters)
        except ValidationError as e:
            raise IncorrectLookupParameters(e)

    def expected_parameters(self):
        return [self.lookup_kwarg_since, self.lookup_kwarg_until]

    def choices(self, cl):
        for title, param_dict in self.links:
            yield {
                'selected': self.date_params == param_dict,
                'query_string': cl.get_query_string(
                    param_dict, [self.field_generic]),
                'display': title,
            }

    def has_output(self):
        return True



import datetime
import time

import jdatetime
from django.forms import widgets
from django.utils import datetime_safe, formats
from django.utils.encoding import smart_str


class jDateInput(widgets.Input):
    input_type = 'text'
    format = None

    def __init__(self, attrs=None, format=None):
        super().__init__(attrs)
        if format:
            self.format = format

    def _format_value(self, value):
        if value is None:
            return ''
        elif hasattr(value, 'strftime'):
            f = smart_str(self.format or formats.get_format('DATE_INPUT_FORMATS')[0])
            return value.strftime(f)

        return value

    def render(self, name, value, attrs=None, renderer=None):
        value = self._format_value(value)
        return super().render(name, value, attrs)

    def _has_changed(self, initial, data):
        # If our field has show_hidden_initial=True, initial will be a string
        # formatted by HiddenInput using formats.localize_input, which is not
        # necessarily the format used for this widget. Attempt to convert it.
        try:
            input_format = formats.get_format('DATE_INPUT_FORMATS')[0]
            initial = jdatetime.date(*time.strptime(initial, input_format)[:3])
        except (TypeError, ValueError):
            pass
        return super()._has_changed(self._format_value(initial), data)


class jDateTimeInput(widgets.Input):
    input_type = 'text'
    format = '%Y-%m-%d %H:%M:%S'     # '2006-10-25 14:30:59'

    def __init__(self, attrs=None, format=None):
        super().__init__(attrs)
        if format:
            self.format = format
            self.manual_format = True
        else:
            self.format = formats.get_format('DATETIME_INPUT_FORMATS')[0]
            self.manual_format = False

    def _format_value(self, value):
        if self.is_localized and not self.manual_format:
            return formats.localize_input(value)
        elif hasattr(value, 'strftime'):
            value = datetime_safe.new_datetime(value)
            return value.strftime(self.format)
        return value

    def _has_changed(self, initial, data):
        # If our field has show_hidden_initial=True, initial will be a string
        # formatted by HiddenInput using formats.localize_input, which is not
        # necessarily the format used for this widget. Attempt to convert it.
        try:
            input_format = formats.get_format('DATETIME_INPUT_FORMATS')[0]
            initial = datetime.datetime(*time.strptime(initial, input_format)[:6])
        except (TypeError, ValueError):
            pass
        return super()._has_changed(self._format_value(initial), data)



from django import forms
from django.contrib.admin import options

from django_jalali.admin import widgets
from django_jalali.db.models import jDateField, jDateTimeField

options.FORMFIELD_FOR_DBFIELD_DEFAULTS[jDateField] = {'widget': widgets.AdminjDateWidget}
options.FORMFIELD_FOR_DBFIELD_DEFAULTS[jDateTimeField] = {
    'form_class': forms.SplitDateTimeField,
    'widget': widgets.AdminSplitjDateTime,
}
# maybe we can use same time
# models.TimeField:       {'widget': widgets.AdminTimeWidget},
# models.DateTimeField: {
# 'form_class': forms.SplitDateTimeField,
# 'widget': widgets.AdminSplitDateTime
# }



"""
To add filtering to your admin interface you need to
import django_jalali.admin.filterspecs in to your admin.py
"""
import jdatetime
from django.contrib.admin.filterspecs import FilterSpec
from django.utils.translation import gettext as _

from django_jalali.db.models import jDateField


class jDateFieldFilterSpec(FilterSpec):
    def __init__(self, f, request, params, model, model_admin,
                 field_path=None):
        super().__init__(f, request, params, model, model_admin, field_path=field_path)
        self.field_generic = '%s__' % self.field_path

        self.date_params = dict([(k, v) for k, v in params.items() if k.startswith(self.field_generic)])

        today = jdatetime.date.today()
        one_week_ago = today - jdatetime.timedelta(days=7)
        # today_str = isinstance(self.field, jmodels.DateTimeField) \
        #            and today.strftime('%Y-%m-%d 23:59:59') \
        #            or today.strftime('%Y-%m-%d')
        today_str = today.strftime('%Y-%m-%d')

        last_day_this_month = 29
        if today.month == 12 and today.isleap():
            last_day_this_month = 30
        else:
            last_day_this_month = jdatetime.j_days_in_month[today.month-1]

        last_day_this_year = 29
        if today.isleap():
            last_day_this_year = 30

        self.links = (
            (_('Any date'), {}),
            (_('Today'), {'%s' % self.field_path: today.strftime('%Y-%m-%d')}),
            (_('Past 7 days'), {
                '%s__gte' % self.field_path: one_week_ago.strftime('%Y-%m-%d'),
                '%s__lte' % self.field_path: today_str
            }),
            (_('This month'),  {
                '%s__gte' % self.field_path: today.replace(day=1).strftime('%Y-%m-%d'),
                '%s__lte' % self.field_path: today.replace(day=last_day_this_month).strftime('%Y-%m-%d'),
            }),
            (_('This year'), {
                '%s__gte' % self.field_path: today.replace(day=1, month=1).strftime('%Y-%m-%d'),
                '%s__lte' % self.field_path: today.replace(
                    day=last_day_this_year, month=12,
                ).strftime('%Y-%m-%d')
            })
        )

    def title(self):
        return self.field.verbose_name

    def choices(self, cl):
        for title, param_dict in self.links:
            yield {'selected': self.date_params == param_dict,
                   'query_string': cl.get_query_string(
                                   param_dict,
                                   [self.field_generic]),
                   'display': title}


register_jdate = True
register_jdatetime = False
for f in FilterSpec.filter_specs:
    if f[0] == type(jDateFieldFilterSpec):
        register_jdate = False

if register_jdate is True:
    FilterSpec.filter_specs.insert(
        0, (lambda f: isinstance(f, jDateField), jDateFieldFilterSpec)
    )



from setuptools import find_packages, setup

setup(
    name='django-jalali',
    version='5.0.0',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    description=("Jalali Date support for Django model and admin interface"),
    url='http://github.com/slashmili/django-jalali',
    download_url='http://github.com/slashmili/django-jalali/tarball/master',
    author='Milad Rastian',
    author_email='eslashmili@gmail.com',
    keywords="django jalali",
    license='Python Software Foundation License',
    platforms='any',
    install_requires=["jdatetime>=3.7.0", "django>=3.2"],
    extras_require={'drf': 'djangorestframework>=3.12'},
    long_description=open('README.rst', encoding="utf-8").read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
    ],
)
